%  Copyright (C) 2002-2004 David Roundy
%
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program; if not, write to the Free Software Foundation,
%  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
\begin{code}
module TouchesFiles ( look_touch, choose_touching,
                      select_touching, deselect_not_touching,
                    ) where
import List ( sort )

import PatchChoices ( PatchChoices, patch_choices,
                      get_first_choice, get_middle_choice, get_last_choice,
                      force_firsts, force_lasts,
                    )
import Patch ( Patch, apply_to_filepath, list_touched_files,
               flatten, join_patches,
             )
\end{code}

\begin{code}
select_touching :: [FilePath] -> PatchChoices -> PatchChoices
select_touching [] pc = pc
select_touching files pc = force_firsts xs pc
    where ct _ [] = []
          ct fs (p:ps) = case look_touch fs p of
                         (True, fs') -> p:ct fs' ps
                         (False, fs') -> ct fs' ps
          xs = ct (map fix files) (get_middle_choice pc ++ get_last_choice pc)

deselect_not_touching :: [FilePath] -> PatchChoices -> PatchChoices
deselect_not_touching [] pc = pc
deselect_not_touching files pc = force_lasts xs pc
    where ct _ [] = []
          ct fs (p:ps) = case look_touch fs p of
                         (True, fs') -> ct fs' ps
                         (False, fs') -> p:ct fs' ps
          xs = ct (map fix files) (get_first_choice pc ++ get_middle_choice pc)

fix :: FilePath -> FilePath
fix f | take 1 (reverse f) == "/" = fix $ reverse $ drop 1 $ reverse f
fix "" = "."
fix "." = "."
fix f = "./" ++ f

choose_touching :: [FilePath] -> Patch -> Patch
choose_touching [] p = p
choose_touching files p =
    join_patches $ get_first_choice $ select_touching files $
                 patch_choices $ flatten p

look_touch :: [FilePath] -> Patch -> (Bool, [FilePath])
look_touch fs p = (orr affects (list_touched_files p) fs
                   || fs' /= fs, fs')
    where orr :: (a -> b -> Bool) -> [a] -> [b] -> Bool
          orr fun (a:as) bs = any (fun a) bs || orr fun as bs
          orr _ [] _ = False
          affects touched f | touched == f = True
          affects t f = case splitAt (length t) f of
                        (t', '/':_) -> t' == t
                        _ -> case splitAt (length f) t of
                             (f', '/':_) -> f' == f
                             _ -> False
          fs' = sort $ map (apply_to_filepath p) fs
\end{code}
